/* global BDF,Ficheform,CodeMirror,Infinity,$$ */

Ficheform.Section = {};

Ficheform.Section.ficheElement = null;

Ficheform.Section.init = function () {
    BDF.$({ficheformEntry: "section"}).each(function (index, element) {
        Ficheform.Section.initSection(element);
    });
};

Ficheform.Section.initSection = function (element) {
    var sectionId = BDF.generateId();
    var $entry = $(element);
    var entryKey = $entry.data("ficheformKey");
    var factorStorageKey =  entryKey + ".factor";
    var syntaxStorageKey = entryKey + ".syntax";
    var textArea = BDF.$($entry, {ficheformRole: "sectiontext"})[0];
    var codeMirrorUnit;
    BDF.$($entry, {ficheformRole: "section-area"}).prepend((BDF.render("ficheform:section-areatoolbar", {
        sectionId: sectionId,
        noTab: Ficheform.getOption("noTab")
    }))).prepend(BDF.render("ficheform:section-codemirrortoolbar", {
        sectionId: sectionId,
        noTab: Ficheform.getOption("noTab")
    }));
    $$(sectionId, "syntaxswitch").change(function () {
        _switchCodeMirror(this.checked);
    });
    $$(sectionId, "increase").click(_increaseArea);
    $$(sectionId, "decrease").click(_decreaseArea);
    $$(sectionId, "zoomedit").click(_zoomEdit);
    _checkLocalStore();

    
    function _checkLocalStore() {
        var localStore = BDF.getMainLocalStorage();
        if (!localStore) {
            return;
        }
        if (localStore.hasOwnProperty(factorStorageKey)) {
            let rows = textArea.rows;
            let factor =  localStore[factorStorageKey];
            if (factor > 0) {
                for(let j = 1; j <= factor; j++) {
                    rows = rows * 2;
                }
            } else if (factor < 0) {
                factor = -factor;
                for(let j = 1; j <= factor; j++) {
                    rows = rows / 2;
                }
            }
            textArea.rows = rows;
        }
        if (localStore.hasOwnProperty(syntaxStorageKey)) {
            if (localStore[syntaxStorageKey]) {
                 $$(sectionId, "syntaxswitch").prop("checked", true).trigger('change');
            }
        }
    }
    
    function _switchCodeMirror(checked) {
        if (!checked) {
            $$(sectionId, "codemirrortoolbar").removeClass("ficheform-section-Active");
            if (codeMirrorUnit) {
                codeMirrorUnit.codeMirror.toTextArea();
                codeMirrorUnit.$ficheblockButtons.empty();
                codeMirrorUnit = null;
                $$(sectionId, "areatoolbar").show();
                _storeSyntax(false);
            }
        } else {
            $$(sectionId, "areatoolbar").hide();
            $$(sectionId, "codemirrortoolbar").addClass("ficheform-section-Active");
            codeMirrorUnit = Ficheform.Section.initSectionCodeMirror($entry, textArea, true);
            codeMirrorUnit.codeMirror.on('change', Ficheform.signalChange);
            _storeSyntax(true);
        }
    }
    
    function _increaseArea() {
        let rows = textArea.rows;
        textArea.rows = rows * 2;
        _storeFactor(true);
    }

    function _decreaseArea() {
        let rows = textArea.rows;
        if (rows < 6) {
            return;
        }
        rows = rows / 2;
        textArea.rows = rows;
        _storeFactor(false);
    }
    
    function _zoomEdit() {
        let url = "edition?page=zoomedit&appelant=" + textArea.id;
        Ficheform.openDialog(url,"fenetre_"+ entryKey,670,570);
    }
    
    function _storeSyntax(isSyntax) {
        var localStore = BDF.getMainLocalStorage();
        if (!localStore) {
            return;
        }
        localStore[syntaxStorageKey] = isSyntax;
        BDF.putMainLocalStorage(localStore);
    }
    
    function _storeFactor(increase) {
        var localStore = BDF.getMainLocalStorage();
        if (!localStore) {
            return;
        }
        let currentValue, newValue;
        if (localStore.hasOwnProperty(factorStorageKey)) {
            currentValue = localStore[factorStorageKey];
        } else {
            currentValue = 0;
        }
        if (increase) {
           newValue = currentValue + 1;
        } else {
            newValue = currentValue -1;
        }
        localStore[factorStorageKey] = newValue;
        BDF.putMainLocalStorage(localStore);
    }
};

Ficheform.Section.initSectionCodeMirror = function (container, textArea, disableOnBlur) {
    var genId = BDF.generateId();
    var $ficheblockButtons = BDF.$(container, {role: "ficheform-ficheblockbuttons"});
    var codeMirror = BDF.toCodeMirror(textArea, {
        mode: 'ficheblock',
        theme: 'bdf',
        lineWrapping: true,
        viewportMargin: Infinity
    });
    var codeMirrorUnit = new Ficheform.Section.CodeMirrorUnit(codeMirror, $ficheblockButtons);
    var disableTimeout = false;
    $ficheblockButtons.html(BDF.render("ficheform:section-ficheblockbuttons", {
        genId: genId
    }));
    codeMirror.on('cursorActivity', _checkToolbar);
    codeMirror.on('focus', _checkToolbar);
    if (disableOnBlur) {
        codeMirror.on('blur', _disableToolbar);
    }
    _init("undo", function () {
        codeMirror.getDoc().undo();
        codeMirror.focus();
        _checkToolbar();
    });
    _init("redo", function () {
        codeMirror.getDoc().redo();
        codeMirror.focus();
        _checkToolbar();
    });
    _init("bold", function () {
        BDF.Ficheblock.wrapSelection(codeMirror, "{{", "}}");
    });
    _init("italic", function () {
        BDF.Ficheblock.wrapSelection(codeMirror, "{", "}");
    });
    _init("bolditalic", function () {
        BDF.Ficheblock.wrapSelection(codeMirror, "{{{", "}}}");
    });
    _init("urlref", function () {
        Ficheform.Section.Overlay.showUrlForm(codeMirror);
    });
    _init("ficheref", function () {
        Ficheform.Section.Overlay.showFicheForm(codeMirror);
    });
    _init("stylechoice", function () {
        Ficheform.Section.Overlay.showStyleForm(codeMirror);
    });
    _init("heading1", function () {
        BDF.Ficheblock.insertCharsAtStart(codeMirror, "#");
    });
    _init("heading2", function () {
        BDF.Ficheblock.insertCharsAtStart(codeMirror, "##");
    });
    _init("heading3", function () {
        BDF.Ficheblock.insertCharsAtStart(codeMirror, "###");
    });
    _init("ul", function () {
        BDF.Ficheblock.insertList(codeMirror);
    });
    _init("zonechoice", function () {
        Ficheform.Section.Overlay.showZoneForm(codeMirror);
    });
    _init("paste", function () {
        Ficheform.Section.Overlay.showPasteAndConvert(codeMirror);
    });
    return codeMirrorUnit;

    
    function _init(key, click) {
        return $$(genId, key).focus(_stopDisable).click(click);
    }
    
    function _checkToolbar() {
        var newEnabled = BDF.Ficheblock.checkEnabled(codeMirror);
        for(let prop in newEnabled) {
            BDF.$($ficheblockButtons, {_element: "button", enableType: prop}).prop("disabled", !newEnabled[prop]);
        }
    };
    
    function  _disableToolbar() {
         if (!disableTimeout) {
            disableTimeout = setTimeout(function() {
                BDF.$($ficheblockButtons, {_element: "button"}).prop("disabled", true);
                disableTimeout = false;
            }, 10);
        }
    }
    
    function _stopDisable() {
        if (disableTimeout) {
            clearTimeout(disableTimeout);
            disableTimeout = false;
        }
    }

};

Ficheform.Section.getUriLinkedToFicheArray = function (role) {
    var resultArray = new Array();
    var $uris;
    if (Ficheform.Section.ficheElement) {
        $uris = BDF.$(Ficheform.Section.ficheElement, {ficheformRole: role});
    } else {
        $uris = BDF.$({ficheformRole: role});
    }
    for(let i = 0, len = $uris.length; i < len; i++) {
        let uri = $($uris[i]).text();
        if (uri.indexOf('/') > -1) {
            if ($.inArray(uri, resultArray) === -1) {
               	resultArray.push(uri);
            }
        }
    }
    return resultArray;
};


Ficheform.Section.CodeMirrorUnit = function (codeMirror, $ficheblockButtons) {
    this.codeMirror = codeMirror;
    this.$ficheblockButtons = $ficheblockButtons;
};
